# (C) Copyright IBM Corp. 2006 - All Rights Reserved.
# DISCLAIMER:
# The following source code is sample code created by IBM Corporation.
# This sample code is provided to you solely for the purpose of assisting you
# in the  use of  the product. The code is provided 'AS IS', without warranty or
# condition of any kind. IBM shall not be liable for any damages arising out of your
# use of the sample code, even if IBM has been advised of the possibility of
# such damages.


# These are a set of modules that can be used by other Python scripts to perform
# common WebSphere Admin works or tasks. The intent is to have this be the repository
# of such modules to among other reasons, improve serviceability. There may be scripts
# who that have the same code as that in here, but are not using this class because 
# we can not change their documentation at this point in time. When that becomes 
# possible, we shall remove the code in those scripts and let them call into this
# class to use the common modules. One such example is workclassoperations.py.

# The arguments in these methods, follow those documented for workclassoperations.py
# in syntax. For example, 'aDefaultAction' follows the syntax of workclassoperations.py
# [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# is a valid input.

# Usage: To use these modules, you must do 2 things in your Python scripts
#    1. Define this variable at the top of your scripts file: ModulePath = $WAS_HOME/bin
#       For example, on Windows: ModulePath = "C:\\WebSphere\\AppServer\\bin\\"
#    2. Add this statement in the main section of script
#          execfile(ModulePath+"XDPYModules.py" )
#   You can call any of the methods in XDPYMdules, for example:
#    print findProxyServer("myODR")
#   For methods that throw the NameError exception, the caller should call them inside of
#   a try block. For example,
#             try:
#                  removeAppRoutingWorkClass(wcName,appName)
#             except NameError, detail:
#                  print detail 

#
# Author: O Michael Atogi
# Date Created: 01/24/2006
#

#
# 
import jarray
from java.lang import System
from com.ibm.ws.classify import Expression
from com.ibm.wsspi.classify import RuleParserException

#
# Gets the protocol type of the form used by the AdminTask command given a
# communication protocol type.
#
# @param aProtocolType The name of the communication protocol type. Supported
#                      values are SOAP, IIOP, JMS, and it defaults to HTTP.
# @return String  Representing the workclass type based on the protocol.
#
def getProtocolForWorkClass(aProtocolType):
   if (aProtocolType == "SOAP"):
      protocolType = "SOAPWORKCLASS"
   elif (aProtocolType == "IIOP"):
      protocolType = "IIOPWORKCLASS"
   elif (aProtocolType == "JMS"):
      protocolType = "JMSWORKCLASS"
   else:
      protocolType = "HTTPWORKCLASS"
   return protocolType 

#
# Finds the generic server cluster in WAS repository.
#
# @param anAction  The name of the generic server cluster to look up.
# @return name    The name of the generic server cluster if it exists or None
#                 if it does not exists.
#
def findGenericServerCluster(anAction):
   gscId = AdminConfig.getid("/GenericServerCluster:"+anAction+"/")
   if (gscId == ""):
       return None
   else:
       return gscId
    
#
# Finds the proxy server in WAS repository.
#
# @param anOdrName   The name of the proxy server or ODR to look up.
# @return name    The name of the proxy server if it exists or None
#                 if it does not exists.
#
def findProxyServer(anOdrName):
   psNames=AdminConfig.list("ProxyServer").split("\n")
   for psName in psNames:
       psName=psName.rstrip()
       if (len(psName) > 0 and psName.find("/"+anOdrName+"|") > -1):
             return psName
   return None 
    
#
# Finds the node in WAS repository.
#
# @param aNode   The name of the node to look up.
# @return name    The name of the node if it exists or None
#                 if it does not exists.
#  
def findNode(aNode):
   nodeId = AdminConfig.getid("/Node:"+aNode+"/")
   if (nodeId == ""):
       return None
   else:
       return nodeId
    
#   vhNames=AdminConfig.list("Node").split("\n")
#   for vhName in vhNames:
#       vhName=vhName.rstrip()
#       if (len(vhName) > 0 ):
#          name=AdminConfig.showAttribute(vhName,"name")
#          if (aNode == name):
#             return name
#   return None
    
#
# Finds the virtual host in WAS repository.
#
# @param aVHost   The name of the virtual host to look up.
# @return name    The name of the virtual host if it exists or None
#                 if it does not exists.
#
def findVirtualHost(aVHost):
   vhostId = AdminConfig.getid("/VirtualHost:"+aVHost+"/")
   if (vhostId == ""):
       return None
   else:
       return vhostId
     
#
# Finds application routing workclass in WAS repository.
#
# @param aWCName  The name of the workclass to look up.
# @param anAppName  The name of the enterprise application for which the workclass belongs.
# @return name    The name of the workclass if it exists or None
#                 if it does not exists.
# 
def findAppRoutingWorkClass(aWCName,anAppName):
   applicationID = findApplication(anAppName)
   
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          appName = anAppName.split("-edition")[0]
          if (aWCName == name and wcName.find("odr") == -1 and wcName.find("/middlewareappeditions/") == -1 and wcName.find("/deployments/") == -1):
              if (wcName.find("/"+appName+".ear/") > 0 or wcName.find("/"+appName+"/") > 0):
                 appName=AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")[0]
                 if (appName.find(anAppName.split("-edition")[0]) > 0 or appName==""):
                    return wcName
   return None
    
#
# Finds the system application routing workclass in WAS repository.
#
# @param aWCName   The name of the system application routing workclass to look up.
# @param anAppName  The name of the enterprise application for which the workclass belongs.
# @return name    The name of the system application routing workclass if it exists or None
#                 if it does not exists.
#
def findSystemAppRoutingWorkClass(aWCName,anAppName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          appName = anAppName.split("-edition")[0]          
          if (aWCName == name and wcName.find(appName+".ear") > 0 and wcName.find("systemApps") > 0 and wcName.find("deployments") == -1):
                return wcName
   return None
    
#
# Finds the system application service policy workclass in WAS repository.
#
# @param aWCName   The name of the system application service policy workclass to look up.
# @param anAppName  The name of the enterprise application for which the workclass belongs.
# @return name    The name of the system application service policy workclass if it exists or 
#                None if it does not exists.
#
def findSystemAppSLAWorkClass(aWCName,anAppName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          appName = anAppName.split("-edition")[0]          
          if (aWCName == name and wcName.find(appName+".ear") > 0 and wcName.find("systemApps") > 0 and wcName.find("deployments") > 0):
                return wcName
   return None
    
#
# Finds the application service policy workclass in WAS repository.
#
# @param aWCName   The name of the application service policy workclass to look up.
# @param anAppName  The name of the enterprise application for which the workclass belongs.
# @return name    The name of the application service policy workclass if it exists or 
#                None if it does not exists.
#
def findAppSLAWorkClass(aWCName,anAppName):
   deploymentID = findDeployment(anAppName)
   #if (deploymentID == None):
   #    return None
   
   wcNames = AdminConfig.list("WorkClass", deploymentID).split("\n")
   appnames=anAppName.split("-edition")
   mwApp = findMiddlewareApplication(anAppName)
   edname = ""
   if (mwApp != None):
      edition=findMiddlewareAppEdition(anAppName)
      edname=AdminConfig.showAttribute(edition,"name")
   for wcName in wcNames:
      wcName=wcName.rstrip()
      if (len(wcName) > 0 ):
         name=AdminConfig.showAttribute(wcName,"name")
         if (aWCName == name and wcName.find("/"+anAppName+"/") > 0 and wcName.find("deployments") > 0):
            return wcName
         elif (aWCName == name and wcName.find("/"+appnames[0]+"/") > 0 and wcName.find("/middlewareappeditions/"+edname+"/") > 0):
            return wcName 
   return None
    
#
# Finds the workclass for a generic server application in WAS repository.
#
# @param aWCName   The name of the workclass to look up.
# @param aPolicyType The policy type for which this workclass is for. Acceptible values are 'routing', 'sla'.
# @param anOdrName  The name of the on demand router server. 
# @return name    The name of the workclass if it exists or None if it does not exists.
#
def findWorkClass(aWCName,aPolicyType,anOdrName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   for wcName in wcNames:
       wcName=wcName.rstrip()
       
       if (len(wcName) > 0 and wcName.find(aPolicyType) > -1 and wcName.find("/"+anOdrName+"/") > 0):
          name=AdminConfig.showAttribute(wcName,"name")
          if (aWCName == name):
             return wcName
   return None
     
    
#
# Finds the transaction class in WAS repository.
#
# @param aTCName   The name of the transaction class to look up.
# @return name    The name of the transaction class if it exists or 
#                None if it does not exists.
# 
def findTransactionClass(aTCName):
   if (aTCName.find("?") > 0):
       raise NameError("ERROR: The transaction class '"+aTCName+"' does not exist.")
   else:   
       tcName = AdminConfig.getid("/TransactionClass:"+aTCName+"/")
   if (tcName == ""):
       return None
   else:
       return tcName


#
# Finds the middleware application edition
#
# @param aAppName   The name of the middleware application to look up.
# @return id        The id of the middleware application if it 
#                   exists or None if it does not exists.
#
def findMiddlewareApplication(aAppName):
   names = aAppName.split("-edition")
   mwApp = AdminConfig.getid("/MiddlewareApp:"+names[0]+"/")
   if (mwApp == ""):
       return None
   else:
       return mwApp


#
# Finds the middleware application edition
#
# @param appName  The name of the middleware application edition
# @return edition The edition if it exists or None if it does not exist
#    
def findMiddlewareAppEdition(appName):
   names = appName.split("-edition")
   mwApp = findApplication(appName)
   if (mwApp != None):
       editions = AdminConfig.list("MiddlewareAppEdition", mwApp).split("\n")
       for edition in editions:
          if (edition.find("/middlewareapps/" + names[0] + "/") > 0):
             alias = AdminConfig.showAttribute(edition, "alias")
             if (alias == names[1]):
                return edition
   return None


#
# Finds the application in WAS repository.
#
# @param aAppName The name of the application to look up.
# @return name    The name of the application if it exists or 
#                 None if it does not exists.
#
def findApplication(aAppName):
   appName=AdminConfig.getid("/Deployment:"+aAppName+"/")
   if (appName == ""):
       appName = findMiddlewareApplication(aAppName)
       if (appName == None):
          print "ERROR: The application '"+aAppName+"' does not exist."
          return None

   appName = appName.replace("\"","") 
   return appName
    
#
# Finds the enterprise application in WAS repository.
#
# @param aAppName   The name of the enterprise application to look up.
# @return name    The name of the enterprise application if it exists or 
#                None if it does not exists.
#
def findDeployment(aAppName):
   appName=AdminConfig.getid("/Deployment:"+aAppName+"/")
   if (appName == ""):
       appName = findMiddlewareAppEdition(aAppName)
       if (appName == None):
          print "ERROR: The application '"+aAppName+"' does not exist."
          return None

   appName = appName.replace("\"","") 
   return appName
    
#
# Finds the enterprise application module in WAS repository.
#
# @param aAppName     The name of the enterprise application to look up.
# @param aModuleName  The name of the module to look up.
# @return name        The name of the enterprise application moduleif it 
#                     exists or None if it does not exists.
#
def findModuleName(aAppName, aModuleName):
   names = aAppName.split("-edition")
   mwApp = findMiddlewareApplication(aAppName)
   if (mwApp == None): 
      if (AdminApp.listModules(aAppName).find("#"+aModuleName+"+") > 0):
         return aModuleName
   else:
       modules = AdminTask.listMiddlewareAppWebModules('[-app '+names[0]+' -edition '+names[1]+']').split("\n")
       for mod in modules:
          if (mod == aModuleName):
              return aModuleName
       return None
       
#
# Finds the system application in WAS repository.
#
# @param aAppName   The name of the system application to look up.
# @return name    The name of the system application if it exists or 
#                None if it does not exists.
#
def findSystemApplication(aAppName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   earAppName=""
   if (aAppName.endswith(".ear")):
       earAppName=aAppName
   else:
       earAppName=aAppName+".ear"
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          if (name.startswith("Default_") and name.endswith("_WC")):
              appName=AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")[0]
              if (appName.find(earAppName) > 0):
                 return earAppName
   return None

#
# Lists workclasses for the given application.
#
# @param aAppName   The name of the system application to look up.
# @return workclasses    The array of workclass names if they exist or 
#                         None if it does not exists.
#
def listApplicationWorkClasses(aAppName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   wcns=jarray.zeros(10,java.lang.String)
   earAppName=""
   if (aAppName.endswith(".ear")):
       earAppName=aAppName
   else:
       mwapp = findMiddlewareApplication(aAppName)
       if (mwapp == None):
           earAppName=aAppName+".ear"
       else:
           earAppName=aAppName
   k=0
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          if (name.startswith("Default_") and name.endswith("_WC")):
              appName=AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")[0]
              if (appName.find(earAppName) > 0):
                 wcns[k]= name
                 k += 1
   if (k == 0):
     return None
   return resize(wcns)

#
# Lists workclass IDs for the given application.
#
# @param aAppName   The name of the system application to look up.
# @return workclasses    The array of workclass names if they exist or 
#                         None if it does not exists.
#
def listApplicationWorkClassIDs(aAppName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   wcns=jarray.zeros(100,java.lang.String)
   earAppName=""
   if (aAppName.endswith(".ear") > 0):
       earAppName=aAppName
   else:
       mwapp = findMiddlewareApplication(aAppName)
       if (mwapp == None):
          earAppName=aAppName+".ear"
       else:
          earAppName=aAppName
   k=0
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          appName=AdminConfig.showAttribute(wcName, "workClassModules").replace("[","").replace("]","").split(" ")[0]
          if (appName.find(earAppName) > 0):
             wcns[k]= wcName
             k += 1
   if (k == 0):
     return None
   return resize(wcns)

#
# Lists generic server workclass IDs for the given ODR.
#
# @param anOdrName      The name of the on demand router to look up.
# @return workclasses   The array of workclass names if they exist or 
#                       None if it does not exists.
#
def listGenericServerWorkClassIDs(anOdrName):
   wcNames=AdminConfig.list("WorkClass").split("\n")
   wcns=jarray.zeros(100,java.lang.String)

   k=0
   for wcName in wcNames:
       wcName=wcName.rstrip()

       if (wcName.find("/"+anOdrName+"/") > 0):
           wcns[k] = wcName
           k += 1

   if (k == 0):
     return None
   return resize(wcns)

#
# Lists workclasses for the given application.
#
# @param aAppName   The name of the system application to look up.
# @return workclasses    The array of workclass names if they exist or 
#                         None if it does not exists.
#
def listSystemAppWorkClassIDs():
   wcNames=AdminConfig.list("WorkClass").split("\n")
   wcns=jarray.zeros(100,java.lang.String)

   k=0
   for wcName in wcNames:
       wcName=wcName.rstrip()
       
       if (wcName.find("xd/systemApps/") > 0):
           wcns[k] = wcName
           k += 1

   if (k == 0):
     return None
   return resize(wcns)

#
# Lists all workclasses in this cell.
#
# @return workclasses    The array of workclass names if they exist or 
#                         None if it does not exists.
#
def listWorkClasses():
   wcNames=AdminConfig.list("WorkClass").split("\n")
   k=0
   wcns=jarray.zeros(200,java.lang.String)
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          name=AdminConfig.showAttribute(wcName,"name")
          wcns[k] = name
          k += 1
   if (k == 0):
     return None
   return resize(wcns)

#
# Lists all workclasses in this cell.
#
# @return workclasses    The array of workclass names if they exist or 
#                         None if it does not exists.
#
def listWorkClassIDs():
   wcNames=AdminConfig.list("WorkClass").split("\n")
   k=0
   wcns=jarray.zeros(200,java.lang.String)
   for wcName in wcNames:
       wcName=wcName.rstrip()
       if (len(wcName) > 0 ):
          wcns[k] = wcName
          k += 1
   if (k == 0):
     return None
   return resize(wcns)

#
# Resizes to create a packed array eliminating empty elements.
#
# @return newarray  A packed array of work class names.
#
def resize(_data):
   count=0
   for j in range(0,len(_data),1):
       if (_data[j] != None):
          count += 1
   wcNames=jarray.zeros(count,java.lang.String)
   for i in xrange(count):
      wcNames[i] = _data[i]
   return wcNames
    
##
# Creates a routing workClass for generic server cluster with user specified workclass name, 
# default action, protocol type, rules, URI patterns, virtual host, and odr node.
#
# @param aVirtualHost   The name of the virtual host.
# @param anOdrNode    The name of the odr node.
# @param anOdrName    The name of the ODR.
# @param aDefaultAction The default action for the workclass. 
#                       The value of aDefaultAction follows the syntax of workclassoperations.py
#                       [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# @param aWorkClassName The name of the workclass to be created.
# @param aProtocolType  The protocol type. Possible values are HTTP, SOAP, JMS, IIOP.
# @param aRule   Additional rules for the workclass. The aRule follows the syntax of workclassoperations.py addRules option; 
#                 therefore, "priority?expr?actiontype?action" is the format for aRule. aRule is an array object.
# @param aMembers   A set of generic server members for the ODR following syntax of workclassoperations.py addMembers option.
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##
def createGSCRoutingWorkClass(aVirtualHost,anOdrNode,anOdrName,aDefaultAction,aWorkClassName,aProtocolType,aRule,aMembers):
   vhostName=findVirtualHost(aVirtualHost)
   odrNode=findNode(anOdrNode)
   odrName=findProxyServer(anOdrName)
   
   if (findWorkClass(aWorkClassName,"routing",anOdrName) != None):
       raise NameError("WARNING: WorkClass already exists; therefore, cannot call create.")

   if (vhostName != None and odrNode != None and odrName != None):
       if (aDefaultAction != ""):
           dParts = aDefaultAction.split("?")
           actionType = dParts[0]
           validateRoutingActionType(actionType)
           actionValue = ""
           try:
              actionValue = dParts[1]
           except IndexError:
              errmsg = "ERROR: missing action for '"+actionType+"'."
              raise NameError(errmsg)               
          
           if (actionType.startswith("permit")):
              gscName=findGenericServerCluster(actionValue)
              if (gscName == None):
                 raise NameError("ERROR: The generic server cluster '"+actionValue+"' does not exist.")

           taskCmdArgs =  "[-odrnode "+ anOdrNode +" -odrname "+ anOdrName+" -wcname "+aWorkClassName+" -type "+aProtocolType+" -actiontype "+actionType+" -action "+actionValue+" -vhost "+aVirtualHost+"]"
           AdminTask.createRoutingPolicyWorkClass(taskCmdArgs)           
           print "INFO: "+aWorkClassName+" work class created."
           addGSCRoutingRule(aWorkClassName,anOdrName,aRule)
           addGSCRoutingMember(aWorkClassName,anOdrNode,anOdrName,aMembers)
           if (aMembers != ""):
               print "INFO: members "+aMembers+" added successfully."
   else:
      if (vhostName == None):
         errMsg = " virtual host."
      elif (odrNode == None):
         errMsg = " ODR node."
      elif (odrName == None):
         errMsg = " ODR."
      else:
         errMsg = "check command arguments and re-issue command."
      errmsg = "ERROR: The specified arguments are not found in this WAS deployment: "+errMsg
      raise NameError(errmsg)
          
##
# Creates a service policy workclass for a generic server cluster with user specified 
# workclass name, default action, protocol type, rules, URI patterns, virtual host, and odr node.
# 
# @param aVirtualHost   The name of the virtual host.
# @param anOdrNode    The name of the odr node.
# @param anOdrName    The name of the ODR.
# @param aDefaultAction The default transaction class for the workclass.
#                       The value of aDefaultAction follows the syntax of workclassoperations.py
#                       [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# @param aWorkClassName The name of the workclass to be created.
# @param aProtocolType  The protocol type. Possible values are HTTP, SOAP, JMS, IIOP.
# @param aRule   Additional rules for the workclass. The aRule follows the syntax of workclassoperations.py addRule option; 
#                 therefore, "priority?expr?actiontype?action" is the format for aRule. aRule is an array object.
# @param aMembers   A set of web module URIs for the application webmodule following syntax of workclassoperations.py addMembers option.
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##
def createGSCSLAWorkClass(aVirtualHost,anOdrNode,anOdrName,aDefaultAction,aWorkClassName,aProtocolType,aRule,aMembers):
   vhostName=findVirtualHost(aVirtualHost)
   odrNode=findNode(anOdrNode)
   odrName=findProxyServer(anOdrName)
   tcName = findTransactionClass(aDefaultAction)
   if (findWorkClass(aWorkClassName,"sla",anOdrName) != None):
       raise NameError("WARNING: WorkClass already exists; therefore, cannot call create.")

   if (vhostName != None and odrNode != None and odrName != None and tcName != None):
       taskCmdArgs =  "[-odrnode "+ anOdrNode +" -odrname "+ anOdrName+" -wcname "+aWorkClassName+" -type "+aProtocolType+" -tcname "+aDefaultAction+" -vhost "+aVirtualHost+"]"
       AdminTask.createServicePolicyWorkClass(taskCmdArgs)
       print "INFO: "+aWorkClassName+" work class created."               

       addGSCSLARule(aWorkClassName,anOdrName,aRule)
       addGSCSLAMember(aWorkClassName,anOdrNode,anOdrName,aMembers)
              
       if (aMembers != ""):         
           print "INFO: members "+aMembers+" added successfully."
   else:
      if (vhostName == None):
         errMsg = " virtual host."
      elif (odrNode == None):
         errMsg = " ODR node."
      elif (odrName == None):
         errMsg = " ODR."
      elif (tcName == None):
         errMsg = " transaction class name."
      else:
         errMsg = "check command arguments and re-issue command."
      errmsg = "ERROR: The specified arguments are not found in this WAS deployment: "+errMsg
      raise NameError(errmsg)
    
##
# Creates an application routing policy workclass using specified arguments. 
# 
# @param anAppName The name of the enterprise application.
# @param aModule The name of the module.
# @param aDefaultAction The defaul action for the workclass.
#                       The value of aDefaultAction follows the syntax of workclassoperations.py
#                       [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# @param aWorkClassName The name of the workclass to be created.
# @param aProtocolType  The protocol type. Possible values are HTTP, SOAP, JMS, IIOP.
# @param aRule   Additional rules for the workclass. The aRule follows the syntax of workclassoperations.py addRules option; 
#                 therefore, "priority?expr?actiontype?action" is the format for aRule. aRule is an array object.
# @param aMembers   A set of web module URIs for the application webmodule following syntax of workclassoperations.py addMembers option.
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##
def createAppRoutingWorkClass(anAppName,aModule,aDefaultAction,aWorkClassName,aProtocolType,aRule,aMembers):
   if (findAppRoutingWorkClass(aWorkClassName,anAppName) != None):
       raise NameError("WARNING: WorkClass already exists; therefore, cannot call create.")
   
   appLongName=findApplication(anAppName)
   appShortName=anAppName

   if (appLongName != None):
       if (aDefaultAction != ""):
           dParts = aDefaultAction.split("?")
           actionType = dParts[0]
           validateRoutingActionType(actionType)
           actionValue = ""
           try:
              actionValue = dParts[1]
           except IndexError:
              errmsg = "ERROR: missing action for '"+actionType+"'."
              raise NameError(errmsg)

           if (actionType.startswith("permit")):
               deploymentName=findDeployment(actionValue)
               if (deploymentName == None):
                   raise NameError("ERROR: The application deployment '"+actionValue+"' does not exist.")
               
           names=anAppName.split("-edition")
           mwApp = findMiddlewareApplication(anAppName)
           if (mwApp == None):
              taskCmdArgs =  '[-appname "'+ anAppName+'" -wcname '+aWorkClassName+' -type '+aProtocolType+' -actiontype '+actionType+' -action "'+actionValue+'"]'
              AdminTask.createRoutingPolicyWorkClass(taskCmdArgs)
           else:
              protocolType = getProtocolForWorkClass(aProtocolType) 
              wclassAttributes = [["matchAction",actionType+":"+actionValue],
                                   ["name",aWorkClassName],
                                   ["type",protocolType],
                                   ["description","This is a custom workclass."]]
                                   
              AdminConfig.create("WorkClass",mwApp,wclassAttributes)
               
           print "INFO: "+aWorkClassName+" work class created."                 
                 
           addAppRoutingRule(aWorkClassName,anAppName,aRule)
           addAppRoutingMembers(aWorkClassName,anAppName,aModule,aMembers)

           if (aMembers != ""):
               print "INFO: members "+aMembers+" added successfully."
   else:
      raise NameError("ERROR: The application '"+appShortName+"' does not exist.")
    
##
# Creates an enterprise application service policy workclass using specified arguments.
# 
# @param anAppName      The name of the enterprise application.
# @param aModule        The name of the web module.
# @param anEJBName      The name of the EJB module.
# @param aDefaultAction The default transaction class for the workclass.
#                       The value of aDefaultAction follows the syntax of workclassoperations.py
#                       [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# @param aWorkClassName The name of the workclass to be created.
# @param aProtocolType  The protocol type. Possible values are HTTP, SOAP, JMS, IIOP.
# @param aRule          Additional rules for the workclass. The aRule follows the syntax of workclassoperations.py addRules option; 
#                       therefore, "priority?expression?actiontype?action" is the format for aRule. aRule is an array object.
# @param aMembers       A set of module members for the application module following syntax of workclassoperations.py addMembers option.
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##
def createAppSLAWorkClass(anAppName,aModule,anEJBName,aDefaultAction,aWorkClassName,aProtocolType,aRule,aMembers):
   if (findAppSLAWorkClass(aWorkClassName,anAppName) != None):
       raise NameError("WARNING: WorkClass already exists; therefore, cannot call create.")
   
   protocolType = getProtocolForWorkClass(aProtocolType)
   appLongName=findDeployment(anAppName)
   
   if (appLongName != None):
       if (aDefaultAction != ""):
           tcName=findTransactionClass(aDefaultAction)
           
           if (tcName != None):
               wclassAttributes = [["matchAction",aDefaultAction],
                                   ["name",aWorkClassName],
                                   ["type",protocolType],
                                   ["description","This is a custom workclass."]]
                                   
               slaWorkClass=AdminConfig.create("WorkClass",appLongName,wclassAttributes)
               print "INFO: "+aWorkClassName+" work class created."
           else:
               raise NameError("ERROR: The transaction class '"+aDefaultAction+"' does not exist.")

           if (protocolType != "JMSWORKCLASS"):
               addAppSLARule(aWorkClassName,anAppName,aRule)
           addAppSLAMembers(aWorkClassName,anAppName,aModule,aMembers,anEJBName)

           if (aMembers != ""):         
              print "INFO: members "+aMembers+" added successfully."
   else:
      raise NameError("ERROR: The application '"+anAppName+"' does not exist.")
       
##
# Removes the specified application routing policy workclass.
#
# @param aWorkClassID a valid work class ID to remove.
##
def removeWorkClass(aWorkClass):
   aWorkClassName=AdminConfig.showAttribute(aWorkClass,"name")
   if (checkForReservedWC(aWorkClassName) == 1):
       raise NameError("ERROR: A default workclass can not be removed")
   
   AdminConfig.remove(aWorkClass)
   
##
# Adds members for the specified application service policy workclass.
#
# @param aWorkClassName The name of the workclass to delete.
# @param anAppName      The name of the enterprise application to which the workclass belongs.
# @param aModule        The name of the web module.
# @param aMembers       A set of module patterns for the application module following syntax of workclassoperations.py -addmember option.
# @param anEJBName      The name of the EJB.
#
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##   
def addAppSLAMembers(aWorkClassName,anAppName,aModule,aMembers,anEJBName):
   slaWorkClass=""
   slaWorkClassModule=""
   slaRules=""
   oldMember=""

   appLongName=findDeployment(anAppName)
   appShortName=anAppName
      
   if (appLongName != None):
       if (aModule != ""):
           moduleNameValue = findModuleName(appShortName, aModule)
           
           if (moduleNameValue == None):
               raise NameError("ERROR: The module '"+aModule+"' does not exist in application "+appShortName+".")
           
           if (anEJBName != ""):
               aMembers = alterMemberHelper(anEJBName, aMembers)
               
           slaWorkClass=findAppSLAWorkClass(aWorkClassName,appShortName)
           addMemberHelper(slaWorkClass, aWorkClassName, appShortName, moduleNameValue, aMembers)
   else:
      raise NameError("ERROR: The application '"+appShortName+"' does not exist.")

##
# Adds members for the specified application routing policy workclass.
#
# @param aWorkClassName The name of the workclass to delete.
# @param anAppName      The name of the enterprise application to which the workclass belongs.
# @param aModule        The name of the web module.
# @param aMembers       A set of module patterns for the application module following syntax of workclassoperations.py -addmember option.
#
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##  
def addAppRoutingMembers(aWorkClassName,anAppName,aModule,aMembers):
   routingWorkClass=""
   routingWorkClassModule=""
   routingRules=""
   actionValue=""
   oldMember=""

   appLongName=findApplication(anAppName)
   appShortName=anAppName

   if (appLongName != None):
       if (aModule != ""):
           moduleNameValue = findModuleName(appShortName, aModule)
           
           if (moduleNameValue == None):
               raise NameError("ERROR: The module '"+aModule+"' does not exist in application "+appShortName+".")
   
           routingWorkClass = findAppRoutingWorkClass(aWorkClassName,appShortName)
           addMemberHelper(routingWorkClass, aWorkClassName, appShortName, moduleNameValue, aMembers)
   else:
      raise NameError("ERROR: The application '"+appShortName+"' does not exist.")

##
# Adds members for the specified generic server cluster service policy workclass.
#
# @param aWorkClassName The name of the workclass to delete.
# @param anOdrNode      The name of the odr node.
# @param anOdrName      The name of the ODR.
# @param aMembers       A set of module patterns for the application module following syntax of workclassoperations.py -addmember option.
#
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##  
def addGSCSLAMember(aWorkClassName,anOdrNode,anOdrName,aMembers):
   odrNode=findNode(anOdrNode)
   odrName=findProxyServer(anOdrName)

   if (odrNode != None and odrName != None):
       slaWorkClass=findWorkClass(aWorkClassName,"sla",anOdrName)
       addMemberHelper(slaWorkClass, aWorkClassName, anOdrNode, anOdrName, aMembers)
       
   else:
      if (odrNode == None):
         errMsg = " ODR node."
      elif (odrName == None):
         errMsg = " ODR."
      else:
         errMsg = "check command arguments and re-issue command."
      errmsg = "ERROR: The arguments are not found in this WAS deployment: "+errMsg
      raise NameError(errmsg)

##
# Adds members for the specified generic server cluster routing policy workclass.
#
# @param aWorkClassName The name of the workclass to delete.
# @param anOdrNode      The name of the odr node.
# @param anOdrName      The name of the ODR.
# @param aMembers       A set of module patterns for the application module following syntax of workclassoperations.py -addmember option.
#
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##
def addGSCRoutingMember(aWorkClassName,anOdrNode,anOdrName,aMembers):
   odrNode=findNode(anOdrNode)
   odrName=findProxyServer(anOdrName)

   if (odrNode != None and odrName != None):
       slaWorkClass=findWorkClass(aWorkClassName,"routing",anOdrName)
       addMemberHelper(slaWorkClass, aWorkClassName, anOdrNode, anOdrName, aMembers)
       
   else:
      if (odrNode == None):
         errMsg = " ODR node."
      elif (odrName == None):
         errMsg = " ODR."
      else:
         errMsg = "check command arguments and re-issue command."
      errmsg = "ERROR: The arguments are not found in this WAS deployment: "+errMsg
      raise NameError(errmsg)

##
# CAUTION: This is a helper method for all the add member methods.  
# This is not intended to be used directly.  Doing so will by pass
# error checking and possibly result in a corrupted configuration.
##
def addMemberHelper(aWorkClass, aWorkClassName, appOrNode, moduleOrODR, aMembers):
   if (aWorkClass != None):
       if (checkForReservedWC(aWorkClassName) == 1):
           raise NameError("WARNING: Cannot add members to a default work class.")
   
       if (aMembers == ""):
           return None
       else:
           memberExpression = ""
           for member in aMembers.split("?"):
               if (memberExpression == ""):
                   memberExpression = member
               else:
                   memberExpression = memberExpression +","+member

       wcmId = ""
       if (memberExpression == "/*"):
           memberExpression = "*"
       fExpression=memberExpression

       wcmList = AdminConfig.showAttribute(aWorkClass, "workClassModules").replace("[","").replace("]","").split(" ")
       for wcm in wcmList:
          if (wcm == ""):
             break

          if (AdminConfig.showAttribute(wcm, "moduleName") == moduleOrODR):
             wcmId = wcm
             break
            
       if (wcmId == ""):
          workclassID = aWorkClassName+":!:"+appOrNode+":!:"+moduleOrODR
          
          tester = findProxyServer(moduleOrODR)
          if (findProxyServer(moduleOrODR) != None):
              #Generic Server work class so switch the vars.
              workclassID = aWorkClassName+":!:"+moduleOrODR+":!:"+appOrNode
         
          wcModuleAttributes = [ ["moduleName",moduleOrODR],
                                 ["matchExpression",memberExpression],
                                 ["id",workclassID]]
                                 
          aWorkClassModule=AdminConfig.create("WorkClassModule",aWorkClass,wcModuleAttributes)
       else:
          oldMember = AdminConfig.showAttribute(wcmId,"matchExpression").replace("[","").replace("]","") 
          fExpression = oldMember + "," +fExpression
          AdminConfig.modify(wcmId,[["matchExpression", fExpression]])
   else: 
       raise NameError("ERROR: The workclass '"+aWorkClassName+"' does not exist.")


##
# This is a helper method that alters the membership if an EJB name is specified
# for JMS work classes.
##
def alterMemberHelper(anEJBName, aMembers):
    anAlteredMembers = ""
    for member in aMembers.split("?"):        
        if (anAlteredMembers == ""):
            anAlteredMembers = anEJBName +":"+ member
        else:
            anAlteredMembers = anAlteredMembers+"?"+anEJBName +":"+ member
            
    return anAlteredMembers

##
# Modifies the default action for the specified application service policy workclass.
#
# @param aWorkClassName   The name of the workclass to delete.
# @param anAppName   The name of the enterprise application to which the workclass belongs.
# @param aDefaultAction The default transaction class for the workclass.
#                       The value of aDefaultAction follows the syntax of workclassoperations.py
#                       [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# @param isSystemApp    system application flag. 0 means false, 1 means true.
#
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##   
def modifyAppSLADefaultAction(aWorkClassName,anAppName,aDefaultAction, isSystemApp):
   appLongName=None
   slaWorkClass=None
   
   if (isSystemApp == 1):
       appLongName = findSystemApplication(anAppName)
   else:
       appLongName=findDeployment(anAppName)       

   if (appLongName != None):
       tcName=findTransactionClass(aDefaultAction)
       
       if (tcName != None):
           if (isSystemApp == 1):
               slaWorkClass = findSystemAppSLAWorkClass(aWorkClassName,anAppName)
           else:
               slaWorkClass = findAppSLAWorkClass(aWorkClassName,anAppName)
               
           if (slaWorkClass != None):
               AdminConfig.modify(slaWorkClass,[["matchAction", aDefaultAction]])
           else:
               raise NameError("ERROR: The workclass '"+aWorkClassName+"' does not exist.")
       else:
           raise NameError("ERROR: The transaction class '"+aDefaultAction+"' does not exist.")
   else:
      raise NameError("ERROR: The application '"+appShortName+"' does not exist.")

##
# Modifies the default action for the specified application routing policy workclass.
#
# @param aWorkClassName   The name of the workclass to delete.
# @param anAppName   The name of the enterprise application to which the workclass belongs.
# @param aDefaultAction The default transaction class for the workclass.
#                       The value of aDefaultAction follows the syntax of workclassoperations.py
#                       [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# @param isSystemApp    system application flag. 0 means false, 1 means true.
#
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##   
def modifyAppRoutingDefaultAction(aWorkClassName,anAppName,aDefaultAction,isSystemApp):
   routingWorkClass=None
   appLongName=None
   appShortName=anAppName

   if (isSystemApp == 1):
       appLongName = findSystemApplication(anAppName)
   else:
       appLongName=findApplication(anAppName)
       
   if (appLongName != None):
       if (aDefaultAction != ""):
           dParts = aDefaultAction.split("?")
           actionType = dParts[0]
           validateRoutingActionType(actionType)
           actionValue = ""
           try:
              actionValue = dParts[1]
           except IndexError:
              errmsg = "ERROR: missing action for '"+actionType+"'."
              raise NameError(errmsg)

           if (actionType.startswith("permit")):
               deploymentName=findDeployment(actionValue)
               if (deploymentName == None):
                   raise NameError("ERROR: The application deployment '"+actionValue+"' does not exist.")

           if (isSystemApp == 1):
               routingWorkClass = findSystemAppRoutingWorkClass(aWorkClassName,anAppName)
           else:
               routingWorkClass = findAppRoutingWorkClass(aWorkClassName,appShortName)
               
           if (routingWorkClass != None):
               AdminConfig.modify(routingWorkClass,[["matchAction", actionType+":"+actionValue]])
           else:            
               raise NameError("ERROR: The workclass '"+aWorkClassName+"' does not exist.")
   else:
      raise NameError("ERROR: The application '"+appShortName+"' does not exist.")

##
# Modifies the default action for the specified generic server cluster service policy workclass.
#
# @param aWorkClassName   The name of the workclass to delete.
# @param anOdrName    The name of the ODR.
# @param aDefaultAction The default transaction class for the workclass.
#                       The value of aDefaultAction follows the syntax of workclassoperations.py
#                       [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##   
def modifyGSCSLADefaultAction(aWorkClassName,anOdrName,aDefaultAction):
   odrName=findProxyServer(anOdrName) 
   if (odrName != None):
       if (aDefaultAction != ""):
           dParts = aDefaultAction.split("?")           
           txclsName = dParts[0]
           tcName=findTransactionClass(txclsName)
           if (tcName != None):   
               slaWorkClass = findWorkClass(aWorkClassName,"sla",anOdrName)
               if (slaWorkClass != None):
                   aMatchAction = AdminConfig.showAttribute(slaWorkClass, "matchAction")
                   mParts = aMatchAction.split(":")
                   vhostName = mParts[0]
                   AdminConfig.modify(slaWorkClass,[["matchAction", vhostName+":"+txclsName]])
               else:
                   raise NameError("ERROR: The workclass '"+aWorkClassName+"' does not exist.")
           else:
               raise NameError("ERROR: The transaction class '"+txclsName+"' does not exist.")
   else:
      raise NameError("ERROR: The ODR '"+anOdrName+ "'` does not exist.")

##
# Modifies the default action for the specified generic server cluster routing policy workclass.
#
# @param aWorkClassName   The name of the workclass to delete.
# @param anOdrName    The name of the ODR.
# @param aDefaultAction The default transaction class for the workclass.
#                       The value of aDefaultAction follows the syntax of workclassoperations.py
#                       [-setdefaultaction ["actiontype?action"]]; therefore, aDefaultAction="permit?StockTrade"
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##   
def modifyGSCRoutingDefaultAction(aWorkClassName,anOdrName,aDefaultAction):
   odrName=findProxyServer(anOdrName)   
   
   if (odrName != None):
       if (aDefaultAction != ""):
           dParts = aDefaultAction.split("?")
           actionType = dParts[0]
           validateRoutingActionType(actionType)
           actionValue = ""
           try:
              actionValue = dParts[1]
           except IndexError:
              errmsg = "ERROR: missing action for '"+actionType+"'."
              raise NameError(errmsg)
          
           if (actionType.startswith("permit")):
              gscName=findGenericServerCluster(actionValue) 
              if (gscName == None):
                 raise NameError("ERROR: The generic server cluster '"+actionValue+"' does not exist.") 

           routingWorkClass = findWorkClass(aWorkClassName,"routing",anOdrName)
           if (routingWorkClass != None):
                 aMatchAction = AdminConfig.showAttribute(routingWorkClass, "matchAction")
                 mParts = aMatchAction.split(":")
                 vhostName = mParts[0]               
                 AdminConfig.modify(routingWorkClass,[["matchAction", vhostName+":"+actionType+":"+actionValue]])
           else:
                 raise NameError("ERROR: The workclass '"+aWorkClassName+"' does not exist.")             
   else:
      raise NameError("ERROR: The ODR '"+anOdrName+ "'` does not exist.")

##
# Modifies the virtual host for the specified generic server cluster service policy workclass.
#
# @param aWorkClassName   The name of the workclass to delete.
# @param anOdrName    The name of the ODR.
# @param aVirtualHost   The name of the virtual host.
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##   
def modifyGSCSLAVirtualHost(aWorkClassName,anOdrName,aVirtualHost):
   vhostName=findVirtualHost(aVirtualHost)    
   odrName=findProxyServer(anOdrName)

   if (odrName != None):
       if (vhostName != None):
           slaWorkClass = findWorkClass(aWorkClassName,"sla",anOdrName)
           if (slaWorkClass != None):
               aMatchAction = AdminConfig.showAttribute(slaWorkClass, "matchAction")
               mParts = aMatchAction.split(":")
               actionType = mParts[1]
               AdminConfig.modify(slaWorkClass,[["matchAction", aVirtualHost+":"+actionType]])
           else:
               raise NameError("ERROR: The workclass '"+aWorkClassName+"' does not exist.")
       else:
           raise NameError("ERROR: The virtual host '"+aVirtualHost+"' does not exist.")           
   else:
      raise NameError("ERROR: The ODR '"+anOdrName+ "'` does not exist.")

##
# Modifies the default action for the specified generic server cluster routing policy workclass.
#
# @param aWorkClassName   The name of the workclass to delete.
# @param anOdrName    The name of the ODR.
# @param aVirtualHost   The name of the virtual host.
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##   
def modifyGSCRoutingVirtualHost(aWorkClassName,anOdrName,aVirtualHost):
   vhostName=findVirtualHost(aVirtualHost)    
   odrName=findProxyServer(anOdrName)
   
   if (odrName != None):
       if (vhostName != None):
           routingWorkClass = findWorkClass(aWorkClassName,"routing",anOdrName)
           if (routingWorkClass != None):
               aMatchAction = AdminConfig.showAttribute(routingWorkClass, "matchAction")
               mParts = aMatchAction.split(":")
               actionType = mParts[1]
               actionValue = mParts[2]
               AdminConfig.modify(routingWorkClass,[["matchAction", aVirtualHost+":"+actionType+":"+actionValue]])
           else:
               raise NameError("ERROR: The workclass '"+aWorkClassName+"' does not exist.")
       else:
           raise NameError("ERROR: The virtual host '"+aVirtualHost+"' does not exist.")
   else:
      raise NameError("ERROR: The ODR '"+anOdrName+ "'` does not exist.")       

##
# Adds rule for the specified application service policy workclass.
#
# @param aWorkClassName The name of the workclass to delete.
# @param anAppName      The name of the enterprise application to which the workclass belongs.
# @param aRule          Additional rules for the workclass. The aRule follows the syntax of workclassoperations.py -addrule option; 
#                       therefore, "priority?expr?actiontype?action" is the format for aRule. aRule is an array object.
#
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##
def addAppSLARule(aWorkClassName,anAppName,aRule):
   slaWorkClass=None
   appShortName=anAppName   
   appLongName=findDeployment(anAppName)       
   
   if (appLongName != None):
       slaWorkClass = findAppSLAWorkClass(aWorkClassName,appShortName)
       
       if (slaWorkClass != None):
           if (AdminConfig.showAttribute(slaWorkClass,"type") == "JMSWORKCLASS"):
               raise NameError("WARNING: Cannot add rules to a JMS work class.")
           for k in range(0,len(aRule),1):
               if (aRule[k] != None):
                   rParts = aRule[k].split("?")
                   priority = rParts[0]
                   validateRulePriority(priority, slaWorkClass)
                   expr = rParts[1]
                   validateExpression(expr)
                   txclsName = rParts[2]
                   #print "priority = "+priority+ " expr="+expr+ " TransactionClass="+txclsName
                   tcName=findTransactionClass(txclsName)
                   if (tcName != None):
                       wcRuleAttributes = [["matchAction",txclsName],
                                           ["matchExpression",expr],
                                           ["priority",priority]]
                       slaRules=AdminConfig.create("MatchRule",slaWorkClass,wcRuleAttributes)
                       print "INFO: rule "+expr+" created in workclass "+aWorkClassName+"."
                   else:
                      raise NameError("ERROR: The transaction class '"+txclsName+"' does not exist.")                       
       else:
          raise NameError("ERROR: The workclass '"+aWorkClassName+"' does not exist.")
   else:
      raise NameError("ERROR: The application '"+appShortName+"' does not exist.")

##
# Adds rule for the specified application routing policy workclass.
#
# @param aWorkClassName The name of the workclass to delete.
# @param anAppName      The name of the enterprise application to which the workclass belongs.
# @param aRule          Additional rules for the workclass. The aRule follows the syntax of workclassoperations.py -addrule option; 
#                       therefore, "priority?expr?actiontype?action" is the format for aRule. aRule is an array object.
#
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
##
def addAppRoutingRule(aWorkClassName,anAppName,aRule):
   routingWorkClass=None
   actionValue=""
   appShortName=anAppName
   appLongName=findApplication(anAppName)

   if (appLongName != None):
       routingWorkClass = findAppRoutingWorkClass(aWorkClassName,appShortName)       
           
       if (routingWorkClass != None):
          for k in range(0,len(aRule),1):
             if (aRule[k] != None):
               #print " the aRule = " + aRule[k] + " and k = %i"  %k
               rParts = aRule[k].split("?")
               priority = rParts[0]
               validateRulePriority(priority, routingWorkClass)               
               #print "priority = " + priority
               expr = rParts[1]
               validateExpression(expr)
               actionType = rParts[2]
               validateRoutingActionType(actionType)
               actionValue = ""
               try:
                  actionValue = rParts[3]
               except IndexError:
                  errmsg = "ERROR: missing action for '"+actionType+"'."
                  raise NameError(errmsg)
              
               if (actionType.startswith("permit")):
                   deploymentName=findDeployment(actionValue)
                   if (deploymentName == None):
                       raise NameError("ERROR: The application deployment '"+actionValue+"' does not exist.")
                   
               #print "priority = "+priority+ " expr="+expr+ " actionType="+actionType + " action="+actionValue
               wcRuleAttributes = [["matchAction",actionType+":"+actionValue],
                                   ["matchExpression",expr],
                                   ["priority",priority]]
               routingRules=AdminConfig.create("MatchRule",routingWorkClass,wcRuleAttributes)
               print "INFO: rule "+expr+" created in workclass "+aWorkClassName+"."               
       else:
          raise NameError("ERROR: The workclass '"+aWorkClassName+"' does not exist.")
   else:
      errmsg = "ERROR: The application '"+appShortName +"' does not exist in this WAS deployment"
      raise NameError(errmsg)

##
# Adds rule for the specified generic server cluster service policy workclass.
#
# @param aWorkClassName The name of the workclass to delete.
# @param anOdrName      The name of the ODR.
# @param aRule          Additional rules for the workclass. The aRule follows the syntax of workclassoperations.py -addrule option; 
#                       therefore, "priority?expr?actiontype?action" is the format for aRule. aRule is an array object.
#
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
## 
def addGSCSLARule(aWorkClassName,anOdrName,aRule):
   odrName=findProxyServer(anOdrName)

   if (odrName != None):       
       slaWorkClass=findWorkClass(aWorkClassName,"sla",anOdrName)
       if (slaWorkClass != None):
          for k in range(0,len(aRule),1):
            if (aRule[k] != None):
               rParts = aRule[k].split("?")
               priority = rParts[0]
               validateRulePriority(priority, slaWorkClass)               
               expr = rParts[1]
               validateExpression(expr)
               actionValue = rParts[2]
               #gscName=findGenericServerCluster(actionValue)  
               tcName=findTransactionClass(actionValue)
               #print "priority = "+priority+ " expr="+expr+ " actionType="+actionType + " action="+actionValue
               if (tcName != None):
                   wcRuleAttributes = [["matchAction",actionValue],
                                       ["matchExpression",expr],
                                       ["priority",priority]]

                   slaRules=AdminConfig.create("MatchRule",slaWorkClass,wcRuleAttributes)
                   print "INFO: rule "+expr+" created in workclass "+aWorkClassName+"."                   
               else:
                   errmsg = "ERROR: The transaction class '"+actionValue +"' does not exist."
                   raise NameError(errmsg)                     
       else:
          errmsg = "ERROR: The workclass '"+aWorkClassName+"' does not exist in this WAS deployment."
          raise NameError(errmsg)
   else:
      errmsg = "ERROR: The ODR '"+anOdrName+"' does not exist in this WAS deployment."
      raise NameError(errmsg)

##
# Adds rule for the specified generic server cluster routing policy workclass.
#
# @param aWorkClassName The name of the workclass to delete.
# @param anOdrName      The name of the ODR.
# @param aRule          Additional rules for the workclass. The aRule follows the syntax of workclassoperations.py -addrule option; 
#                       therefore, "priority?expr?actiontype?action" is the format for aRule. aRule is an array object.
#
# @exception NameError  Is raised for bad input. The message is preceeded with either WARNING: or ERROR: to indicate severity.
## 
def addGSCRoutingRule(aWorkClassName,anOdrName,aRule):
   odrName=findProxyServer(anOdrName)
   workclass = aWorkClassName

   if (odrName != None):
       routingWorkClass=findWorkClass(aWorkClassName,"routing",anOdrName)
       if (routingWorkClass != None):
          for k in range(0,len(aRule),1):
            if (aRule[k] != None):
               rParts = aRule[k].split("?")
               priority = rParts[0]
               validateRulePriority(priority, routingWorkClass)
               expr = rParts[1]
               validateExpression(expr)               
               actionType = rParts[2]
               validateRoutingActionType(actionType)
               
               try:
                  actionValue = rParts[3]
               except IndexError:
                  errmsg = "ERROR: missing action for '"+actionType+"'."
                  raise NameError(errmsg)
               if (actionType.startswith("permit")):
                   gscName=findGenericServerCluster(actionValue)
                   if (gscName == None):
                       errmsg = "ERROR: The generic server cluster '"+actionValue +"' is not valid."
                       raise NameError(errmsg)
               #print "priority = "+priority+ " expr="+expr+ " actionType="+actionType + " action="+actionValue
               wcRuleAttributes = [["matchAction",actionType+":"+actionValue],
                                   ["matchExpression",expr],
                                   ["priority",priority]]
               routingRules=AdminConfig.create("MatchRule",routingWorkClass,wcRuleAttributes)
               print "INFO: rule "+expr+" created in workclass "+aWorkClassName+"."               
       else:
          errmsg = "ERROR: The workclass '"+aWorkClassName +"' does not exist in this WAS deployment, so no other operation was performed."
          raise NameError(errmsg)               
   else:
      errmsg = "ERROR: The ODR '"+anOdrName+"' does not exist in this WAS deployment."
      raise NameError(errmsg)

def checkForReservedWC(aWorkClass):
    wcid = aWorkClass.split("(cells/")
    if (wcid[0].startswith("Default_") and wcid[0].endswith("_WC")):
        return 1

    return 0
                   
##
# Validates that a specified priority is not already taken with in a specified
# work class ID.
# 
# @param aPriority the priority represented by an integer
# @param aWorkClass the work class ID to check for taken rule priority
# @NameError this is thrown if the priority is already taken.
##
def validateRulePriority(aPriority, aWorkClass):
    rules = AdminConfig.showAttribute(aWorkClass,"matchRules").replace("[","").replace("]","").split(" ")
    for k in range(0,len(rules),1):
        if (rules[k] == None or rules[k] == ""):
            break
        
        priority = AdminConfig.showAttribute(rules[k],"priority")
        matchExpression = AdminConfig.showAttribute(rules[k],"matchExpression")
        if (aPriority == priority):
            errmsg = "ERROR: the rule priority "+priority+" is already assigned to rule: "+matchExpression
            raise NameError(errmsg)
    
def validateRoutingActionType(aActionType):
    if (aActionType != "permit" and aActionType != "permitsticky" and aActionType != "reject" and aActionType != "redirect"):
        if (aActionType.find(":") > 0):
            errmsg = "ERROR: invalid action type '"+aActionType+"'. Try "+aActionType.replace(":","?")+" instead."
            raise NameError(errmsg)            
        else:
            errmsg = "ERROR: invalid action type '"+aActionType+"'.  Valid values are permit, permitsticky, reject and redirect."
            raise NameError(errmsg)            

    
def validateExpression(anExpr):
    try:
        Expression(anExpr)
    except RuleParserException, msg:
        errmsg = "ERROR: The rule expression is invalid: ", msg
        raise NameError(errmsg)
                      
#def normalExit(self):
#    java.lang.System.exit(0)
 
#def errorExit(self):
#    java.lang.System.exit(1)
